@@ -1,7 +1,3 @@
-import sys
-import os
-path = os.getcwd()
-sys.path.append(path)
 import gloce.func
 import gloce.area
 import gloce.interpolation
@@ -14,6 +10,11 @@ nanaverage = gloce.func.nanaverage
 arrayinfo = gloce.func.arrayinfo
 fslice = gloce.func.fslice
 exclude_outlier = gloce.func.exclude_outlier
+r2a = gloce.func.r2a
+n2a = gloce.func.n2a
+func_to_landpoint = gloce.func.func_to_landpoint
+func_to_ilatilon = gloce.func.func_to_ilatilon
+func_to_latlon = gloce.func.func_to_latlon
 #------------
 
 #----resample----
