@@ -1,10 +1,13 @@
 import numpy as np
-from gloce.func import nanaverage
+from scipy import stats
 
-def upscaling_2d(matrix, rstep, cstep, func=np.mean, weights=9999,return_w=False):
+
+def upscaling_2d(matrix, rstep, cstep, func=np.mean, weights=9999):
     '''
     Resample 2d-array data.
-
+    220602
+    If the func is not np.nanmean or stats.mode, 
+    the old algorithm from Li Wei is faster actually (4 times faster).
     Parameters
     ----------
     matrix : 2d-array
@@ -28,18 +31,30 @@ def upscaling_2d(matrix, rstep, cstep, func=np.mean, weights=9999,return_w=False
     '''
     row, col = np.shape(matrix)
     temp = matrix.reshape(int(row / rstep), rstep, int(col / cstep), cstep)
-    if np.all(weights==9999):
-        temp = func(temp, axis=1)
-        temp = func(temp, axis=2)
-    else:
-        weights = weights.reshape(int(row / rstep), rstep, int(col / cstep), cstep)
-        temp = func(temp, axis=1,weights = weights)
-        weights = np.nansum(weights, axis=1)
-        temp = func(temp, axis=2,weights = weights)
-        weights = np.nansum(weights, axis=2)
+    if func==np.nanmean or func==stats.mode:
+        temp = np.transpose(temp,[1,3,0,2])
+        temp = temp.reshape(rstep*cstep,int(row / rstep),int(col / cstep))
+        if np.all(weights==9999):
+            temp = func(temp, axis=0)
+        else:
+            weights = weights.reshape(int(row / rstep), rstep, int(col / cstep), cstep)
+            weights = np.transpose(weights,[1,3,0,2])
+            weights = weights.reshape(rstep*cstep,int(row / rstep),int(col / cstep))
+            temp = func(temp, axis=0,weights = weights)
+    else: # func = np.sum/np.nansum/np.mean
+        if np.all(weights==9999):
+            temp = func(temp, axis=1)
+            temp = func(temp, axis=2)
+        else:
+            weights = weights.reshape(int(row / rstep), rstep, int(col / cstep), cstep)
+            temp = func(temp, axis=1,weights = weights)
+            weights = np.nansum(weights, axis=1)
+            temp = func(temp, axis=2,weights = weights)
+            #weights = np.nansum(weights, axis=2)
     return temp
 
 
+
 def upscaling(matrix, rstep, cstep, func=np.mean, weights=9999,return_w=False):
     '''
     Resample data with dimension less than 5 to reduce resolution.
@@ -53,7 +68,7 @@ def upscaling(matrix, rstep, cstep, func=np.mean, weights=9999,return_w=False):
     cstep : int
         Scaling factor for colunm.
     func : TYPE, optional
-        DESCRIPTION. The default is np.mean.
+        DESCRIPTION. The default is np.mean. (stats.mode for modal number)
     weights : ndarray, optional
         Weights that should have the same shape as matrix. The default is 9999.
     return_w : bool, optional
@@ -87,12 +102,12 @@ def upscaling(matrix, rstep, cstep, func=np.mean, weights=9999,return_w=False):
     if len(matrix.shape)>4:
         raise ValueError('The dimension of input data should be less than 5.')
     return temp
-        
+
 
 def downscaling(matrix, rstep, cstep):
     '''
-    
-
+    220602
+    This is much faster than previous version.
     Parameters
     ----------
     matrix : TYPE
@@ -109,17 +124,10 @@ def downscaling(matrix, rstep, cstep):
 
     '''
     row, col = np.shape(matrix)
-    temp = np.ravel(matrix)
-    #Reduce the dimension of the array to one dimension
-    temp = np.array([temp[int(i / cstep):int(i / cstep) + 1]
-        for i in range(cstep * len(temp))])  #Operate horizontally
-    temp = temp.reshape(row, int(col * cstep))
-    temp = temp.T
-    temp = np.ravel(temp)
-    temp = np.array([temp[int(i / rstep):int(i / rstep) + 1]
-        for i in range(rstep * len(temp))])  #Operate vetically
-    temp = temp.reshape(int(col * cstep), int(row * rstep))
-    temp = temp.T
+    temp = np.zeros((row*rstep,col*cstep))
+    for i in range(row):
+        for j in range(col):
+            temp[i*rstep:(i+1)*rstep,j*cstep:(j+1)*cstep]=matrix[i,j]
     return temp
 
     
\ No newline at end of file
